@@ -1,7 +1,10 @@
 from copy import deepcopy
+from enum import Enum
+from functools import lru_cache
+from inspect import isclass
 import json
 from pathlib import Path
-from typing import Any, Dict, Iterable, TypeVar, cast
+from typing import Any, Dict, Iterable, Optional, TypeVar, Union, cast, get_args
 
 import jsonschema
 import pydantic
@@ -15,7 +18,13 @@ from .consts import (
 )
 from .exceptions import JsonschemaValidationError, PydanticValidationError
 from . import models
-from .utils import _ensure_newline, version2tuple
+from .utils import (
+    TransitionalGenerateJsonSchema,
+    _ensure_newline,
+    sanitize_value,
+    strip_top_level_optional,
+    version2tuple,
+)
 
 schema_map = {
     "Dandiset": "dandiset.json",
@@ -25,7 +34,7 @@ schema_map = {
 }
 
 
-def generate_context():
+def generate_context() -> dict:
     import pydantic
 
     field_preamble = {
@@ -53,16 +62,16 @@ def generate_context():
         "PATO": "http://purl.obolibrary.org/obo/PATO_",
         "spdx": "http://spdx.org/licenses/",
     }
-    fields = {}
+    fields: Dict[str, Any] = {}
     for val in dir(models):
         klass = getattr(models, val)
-        if not isinstance(klass, pydantic.main.ModelMetaclass):
+        if not isclass(klass) or not issubclass(klass, pydantic.BaseModel):
             continue
         if hasattr(klass, "_ldmeta"):
-            if "nskey" in klass._ldmeta:
+            if "nskey" in klass._ldmeta.default:
                 name = klass.__name__
-                fields[name] = f'{klass._ldmeta["nskey"]}:{name}'
-        for name, field in klass.__fields__.items():
+                fields[name] = f'{klass._ldmeta.default["nskey"]}:{name}'
+        for name, field in klass.model_fields.items():
             if name == "id":
                 fields[name] = "@id"
             elif name == "schemaKey":
@@ -70,15 +79,42 @@ def generate_context():
             elif name == "digest":
                 fields[name] = "@nest"
             elif name not in fields:
-                if "nskey" in field.field_info.extra:
-                    fields[name] = {"@id": field.field_info.extra["nskey"] + ":" + name}
+                if (
+                    isinstance(field.json_schema_extra, dict)
+                    and "nskey" in field.json_schema_extra
+                ):
+                    fields[name] = {
+                        "@id": cast(str, field.json_schema_extra["nskey"]) + ":" + name
+                    }
                 else:
                     fields[name] = {"@id": "dandi:" + name}
-                if "List" in str(field.outer_type_):
+
+                # The annotation without the top-level optional
+                stripped_annotation = strip_top_level_optional(field.annotation)
+
+                # Using stringification to detect present of list in annotation is not
+                # ideal, but it works for now. A better solution should be used in the
+                # future.
+                if "list" in str(stripped_annotation).lower():
                     fields[name]["@container"] = "@set"
+
+                    # Handle the case where the type of the element of a list is
+                    # an Enum type
+                    type_args = get_args(stripped_annotation)
+                    if (
+                        len(type_args) == 1
+                        and isclass(type_args[0])
+                        and issubclass(type_args[0], Enum)
+                    ):
+                        fields[name]["@type"] = "@id"
+
                 if name == "contributor":
                     fields[name]["@container"] = "@list"
-                if "enum" in str(field.type_) or name == "url":
+                if (
+                    isclass(stripped_annotation)
+                    and issubclass(stripped_annotation, Enum)
+                    or name in ["url", "hasMember"]
+                ):
                     fields[name]["@type"] = "@id"
 
     for item in models.DigestType:
@@ -90,13 +126,20 @@ def generate_context():
     return {"@context": field_preamble}
 
 
-def publish_model_schemata(releasedir: str) -> Path:
+def publish_model_schemata(releasedir: Union[str, Path]) -> Path:
     version = models.get_schema_version()
     vdir = Path(releasedir, version)
     vdir.mkdir(exist_ok=True, parents=True)
     for class_, filename in schema_map.items():
         (vdir / filename).write_text(
-            _ensure_newline(getattr(models, class_).schema_json(indent=2))
+            _ensure_newline(
+                json.dumps(
+                    getattr(models, class_).model_json_schema(
+                        schema_generator=TransitionalGenerateJsonSchema
+                    ),
+                    indent=2,
+                )
+            )
         )
     (vdir / "context.json").write_text(
         _ensure_newline(json.dumps(generate_context(), indent=2))
@@ -104,10 +147,20 @@ def publish_model_schemata(releasedir: str) -> Path:
     return vdir
 
 
-def _validate_obj_json(data, schema, missing_ok=False):
-    validator = jsonschema.Draft7Validator(
-        schema, format_checker=jsonschema.draft7_format_checker
-    )
+def _validate_obj_json(data: dict, schema: dict, missing_ok: bool = False) -> None:
+    validator: Union[jsonschema.Draft202012Validator, jsonschema.Draft7Validator]
+
+    if version2tuple(data["schemaVersion"]) >= version2tuple("0.6.5"):
+        # schema version 0.7.0 and above is produced with Pydantic V2
+        # which is compliant with JSON Schema Draft 2020-12
+        validator = jsonschema.Draft202012Validator(
+            schema, format_checker=jsonschema.Draft202012Validator.FORMAT_CHECKER
+        )
+    else:
+        validator = jsonschema.Draft7Validator(
+            schema, format_checker=jsonschema.Draft7Validator.FORMAT_CHECKER
+        )
+
     error_list = []
     for error in sorted(validator.iter_errors(data), key=str):
         if missing_ok and "is a required property" in error.message:
@@ -117,21 +170,35 @@ def _validate_obj_json(data, schema, missing_ok=False):
         raise JsonschemaValidationError(error_list)
 
 
-def _validate_dandiset_json(data: dict, schema_dir: str) -> None:
+def _validate_dandiset_json(data: dict, schema_dir: Union[str, Path]) -> None:
     with Path(schema_dir, "dandiset.json").open() as fp:
         schema = json.load(fp)
     _validate_obj_json(data, schema)
 
 
-def _validate_asset_json(data: dict, schema_dir: str) -> None:
+def _validate_asset_json(data: dict, schema_dir: Union[str, Path]) -> None:
     with Path(schema_dir, "asset.json").open() as fp:
         schema = json.load(fp)
     _validate_obj_json(data, schema)
 
 
+@lru_cache
+def _get_schema(schema_version: str, schema_name: str) -> Any:
+    r = requests.get(
+        "https://raw.githubusercontent.com/dandi/schema/"
+        f"master/releases/{schema_version}/{schema_name}"
+    )
+    r.raise_for_status()
+    return r.json()
+
+
 def validate(
-    obj, schema_version=None, schema_key=None, missing_ok=False, json_validation=False
-):
+    obj: dict,
+    schema_version: Optional[str] = None,
+    schema_key: Optional[str] = None,
+    missing_ok: bool = False,
+    json_validation: bool = False,
+) -> None:
     """Validate object using pydantic
 
     Parameters
@@ -173,18 +240,16 @@ def validate(
     if json_validation:
         if schema_version == DANDI_SCHEMA_VERSION:
             klass = getattr(models, schema_key)
-            schema = klass.schema()
+            schema = klass.model_json_schema(
+                schema_generator=TransitionalGenerateJsonSchema
+            )
         else:
             if schema_key not in schema_map:
                 raise ValueError(
                     "Only dandisets and assets can be validated "
                     "using json schema for older versions"
                 )
-            schema_filename = schema_map[schema_key]
-            schema = requests.get(
-                f"https://raw.githubusercontent.com/dandi/schema/"
-                f"master/releases/{schema_version}/{schema_filename}"
-            ).json()
+            schema = _get_schema(schema_version, schema_map[schema_key])
         _validate_obj_json(obj, schema, missing_ok)
     klass = getattr(models, schema_key)
     try:
@@ -192,14 +257,16 @@ def validate(
     except pydantic.ValidationError as exc:
         messages = []
         for el in exc.errors():
-            if not missing_ok or el["msg"] != "field required":
+            if not missing_ok or el["type"] != "missing":
                 messages.append(el)
         if messages:
-            raise PydanticValidationError(messages)
+            raise PydanticValidationError(messages)  # type: ignore[arg-type]
 
 
 def migrate(
-    obj: dict, to_version: str = DANDI_SCHEMA_VERSION, skip_validation=False
+    obj: dict,
+    to_version: Optional[str] = DANDI_SCHEMA_VERSION,
+    skip_validation: bool = False,
 ) -> dict:
     """Migrate dandiset metadata object to new schema"""
     obj = deepcopy(obj)
@@ -217,10 +284,7 @@ def migrate(
     if version2tuple(schema_version) > version2tuple(to_version):
         raise ValueError(f"Cannot migrate from {schema_version} to lower {to_version}.")
     if not (skip_validation):
-        schema = requests.get(
-            f"https://raw.githubusercontent.com/dandi/schema/"
-            f"master/releases/{schema_version}/dandiset.json"
-        ).json()
+        schema = _get_schema(schema_version, "dandiset.json")
         _validate_obj_json(obj, schema)
     if version2tuple(schema_version) < version2tuple("0.6.0"):
         for val in obj.get("about", []):
@@ -234,7 +298,7 @@ def migrate(
                 val["schemaKey"] = "AccessRequirements"
         for resource in obj.get("relatedResource", []):
             resource["schemaKey"] = "Resource"
-        if "schemaKey" not in obj.get("assetsSummary"):
+        if "schemaKey" not in obj["assetsSummary"]:
             obj["assetsSummary"]["schemaKey"] = "AssetsSummary"
         if "schemaKey" not in obj:
             obj["schemaKey"] = "Dandiset"
@@ -246,10 +310,10 @@ _stats_var_type = TypeVar("_stats_var_type", int, list)
 _stats_type = Dict[str, _stats_var_type]
 
 
-def _get_samples(value, stats, hierarchy):
+def _get_samples(value: dict, stats: _stats_type, hierarchy: Any) -> _stats_type:
     if "sampleType" in value:
         sampletype = value["sampleType"]["name"]
-        obj = value["identifier"].replace("_", "-")
+        obj = sanitize_value(value["identifier"])
         if obj not in stats[sampletype]:
             stats[sampletype].append(obj)
     if "wasDerivedFrom" in value:
@@ -293,7 +357,7 @@ def _add_asset_to_stats(assetmeta: Dict[str, Any], stats: _stats_type) -> None:
                 if value["species"] not in stats["species"]:
                     stats["species"].append(value["species"])
             if value.get("identifier", None):
-                subject = value["identifier"].replace("_", "-")
+                subject = sanitize_value(value["identifier"])
                 if subject not in stats["subjects"]:
                     stats["subjects"].append(subject)
 
@@ -316,13 +380,18 @@ def _add_asset_to_stats(assetmeta: Dict[str, Any], stats: _stats_type) -> None:
                 stats["tissuesample"].append(sample)
 
     stats["dataStandard"] = stats.get("dataStandard", [])
+
+    def add_if_missing(standard: dict) -> None:
+        if standard not in stats["dataStandard"]:
+            stats["dataStandard"].append(standard)
+
     if "nwb" in assetmeta["encodingFormat"]:
-        if models.nwb_standard not in stats["dataStandard"]:
-            stats["dataStandard"].append(models.nwb_standard)
+        add_if_missing(models.nwb_standard)
     # TODO: RF assumption that any .json implies BIDS
-    if set(Path(assetmeta["path"]).suffixes).intersection((".json", ".nii")):
-        if models.bids_standard not in stats["dataStandard"]:
-            stats["dataStandard"].append(models.bids_standard)
+    if Path(assetmeta["path"]).name == "dataset_description.json":
+        add_if_missing(models.bids_standard)
+    if Path(assetmeta["path"]).suffixes == [".ome", ".zarr"]:
+        add_if_missing(models.ome_ngff_standard)
 
 
 # TODO?: move/bind such helpers as .from_metadata or alike within
@@ -340,4 +409,4 @@ def aggregate_assets_summary(metadata: Iterable[Dict[str, Any]]) -> dict:
         len(stats.pop("tissuesample", [])) + len(stats.pop("slice", []))
     ) or None
     stats["numberOfCells"] = len(stats.pop("cell", [])) or None
-    return models.AssetsSummary(**stats).json_dict()
+    return models.AssetsSummary(**stats).model_dump(mode="json", exclude_none=True)
