@@ -1,6 +1,7 @@
 from hashlib import md5, sha256
 import json
 from pathlib import Path
+from typing import Any, Dict, Optional, Sequence, Set
 
 import pytest
 
@@ -20,11 +21,11 @@ METADATA_DIR = Path(__file__).with_name("data") / "metadata"
 
 
 @pytest.fixture(scope="module")
-def schema_dir(tmp_path_factory):
+def schema_dir(tmp_path_factory: pytest.TempPathFactory) -> Path:
     return publish_model_schemata(tmp_path_factory.mktemp("schema_dir"))
 
 
-def test_asset(schema_dir):
+def test_asset(schema_dir: Path) -> None:
     with (METADATA_DIR / "asset_001.json").open() as fp:
         data_as_dict = json.load(fp)
     # overload (here and below) schemaVersion until we support automagic schema migrations
@@ -34,15 +35,21 @@ def test_asset(schema_dir):
     validate(data_as_dict)
 
 
-def test_dandiset(schema_dir):
+def test_dandiset(schema_dir: Path) -> None:
     with (METADATA_DIR / "meta_000004.json").open() as fp:
         data_as_dict = json.load(fp)
     data_as_dict["schemaVersion"] = DANDI_SCHEMA_VERSION
     _validate_dandiset_json(data_as_dict, schema_dir)
 
 
+def test_id(schema_dir: Path) -> None:
+    with open(schema_dir / "context.json") as fp:
+        context = json.load(fp)
+    assert context["@context"]["hasMember"]["@type"] == "@id"
+
+
 @skipif_no_network
-def test_pydantic_validation(schema_dir):
+def test_pydantic_validation(schema_dir: Path) -> None:
     with (METADATA_DIR / "meta_000004.json").open() as fp:
         data_as_dict = json.load(fp)
     data_as_dict["schemaVersion"] = "0.4.4"
@@ -55,7 +62,7 @@ def test_pydantic_validation(schema_dir):
 
 
 @skipif_no_network
-def test_json_schemakey_validation():
+def test_json_schemakey_validation() -> None:
     with pytest.raises(JsonschemaValidationError) as exc:
         validate(
             {"identifier": "DANDI:000000", "schemaVersion": "0.4.4"},
@@ -75,7 +82,7 @@ def test_json_schemakey_validation():
         ("0.4.2", "PublishedAsset"),
     ],
 )
-def test_mismatch_key(schema_version, schema_key):
+def test_mismatch_key(schema_version: str, schema_key: str) -> None:
     with pytest.raises(ValueError):
         validate({}, schema_version=schema_version, schema_key=schema_key)
 
@@ -149,6 +156,7 @@ def test_mismatch_key(schema_version, schema_key):
                     {
                         "schemaKey": "Person",
                         "name": "Last, first",
+                        "email": "nobody@example.com",
                         "roleName": ["dcite:ContactPerson"],
                     }
                 ],
@@ -264,7 +272,9 @@ def test_mismatch_key(schema_version, schema_key):
         ),
     ],
 )
-def test_requirements(obj, schema_key, missingfields):
+def test_requirements(
+    obj: Dict[str, Any], schema_key: Optional[str], missingfields: Set[str]
+) -> None:
     with pytest.raises(ValueError):
         validate(obj, schema_key=schema_key)
     with pytest.raises(PydanticValidationError) as exc:
@@ -278,7 +288,7 @@ def test_requirements(obj, schema_key, missingfields):
         (
             {"schemaKey": "Dandiset", "schemaVersion": "0.4.4"},
             None,
-            {"field required"},
+            {"Field required"},
             10,
         ),
         (
@@ -288,16 +298,18 @@ def test_requirements(obj, schema_key, missingfields):
                 "schemaVersion": "0.4.4",
             },
             None,
-            {"field required"},
+            {"Field required"},
             9,
         ),
     ],
 )
-def test_missing_ok(obj, schema_key, errors, num_errors):
+def test_missing_ok(
+    obj: Dict[str, Any], schema_key: Optional[str], errors: Set[str], num_errors: int
+) -> None:
     validate(
         obj, schema_key=schema_key, schema_version=DANDI_SCHEMA_VERSION, missing_ok=True
     )
-    with pytest.raises(ValueError) as exc:
+    with pytest.raises(PydanticValidationError) as exc:
         validate(obj, schema_key=schema_key, schema_version=DANDI_SCHEMA_VERSION)
     exc_errors = [el["msg"] for el in exc.value.errors]
     assert len(exc_errors) == num_errors
@@ -305,7 +317,7 @@ def test_missing_ok(obj, schema_key, errors, num_errors):
 
 
 @skipif_no_network
-def test_missing_ok_error():
+def test_missing_ok_error() -> None:
     with pytest.raises(JsonschemaValidationError):
         validate(
             {
@@ -336,13 +348,13 @@ def test_missing_ok_error():
         ({"schemaVersion": "0.6.0"}, "0.5.2"),
     ],
 )
-def test_migrate_errors(obj, target):
+def test_migrate_errors(obj: Dict[str, Any], target: Optional[str]) -> None:
     with pytest.raises(ValueError):
         migrate(obj, to_version=target, skip_validation=True)
 
 
 @skipif_no_network
-def test_migrate_044(schema_dir):
+def test_migrate_044(schema_dir: Path) -> None:
     with (METADATA_DIR / "meta_000004old.json").open() as fp:
         data_as_dict = json.load(fp)
     with pytest.raises(ValueError):
@@ -486,12 +498,12 @@ def test_migrate_044(schema_dir):
         ),
     ],
 )
-def test_aggregate(files, summary):
+def test_aggregate(files: Sequence[str], summary: Dict[str, Any]) -> None:
     metadata = (json.loads((METADATA_DIR / f).read_text()) for f in files)
     assert aggregate_assets_summary(metadata) == summary
 
 
-def test_aggregate_norecord():
+def test_aggregate_norecord() -> None:
     assert aggregate_assets_summary([]) == {
         "numberOfBytes": 0,
         "numberOfFiles": 0,
@@ -502,7 +514,7 @@ def test_aggregate_norecord():
 @pytest.mark.parametrize(
     "version", ["0.1.0", DANDI_SCHEMA_VERSION.rsplit(".", 1)[0], "10000.0.0"]
 )
-def test_aggregate_nonsupported(version):
+def test_aggregate_nonsupported(version: str) -> None:
     with pytest.raises(ValueError) as exc:
         aggregate_assets_summary([{"schemaVersion": version}])
     assert "Allowed are" in str(exc)
@@ -511,7 +523,7 @@ def test_aggregate_nonsupported(version):
 
 
 @skipif_no_network
-def test_validate_older():
+def test_validate_older() -> None:
     with pytest.raises(ValueError):
         validate(
             {"schemaVersion": "0.5.2", "schemaKey": "Anykey"}, json_validation=True
@@ -525,18 +537,16 @@ def test_validate_older():
         )
 
 
-def test_aggregation_bids():
+def test_aggregation_bids() -> None:
     data = [
         {
             "id": "dandiasset:6668d37f-e842-4b73-8c20-082a1dd0d31a",
             "path": "sub-MITU01/ses-20210703h01m05s04/microscopy/sub-MITU01_"
-            "run-1_sample-163_stain-YO_chunk-5_spim.h5",
+            "run-1_sample-163_stain-YO_chunk-5_spim.ome.zarr",
             "access": [
                 {"status": "dandi:OpenAccess", "schemaKey": "AccessRequirements"}
             ],
             "digest": {
-                "dandi:sha2-256": "a91db138c2842c9d3f5bbe30497065c6293dcab0b77"
-                "f774a917ab09fb1fd8eef",
                 "dandi:dandi-etag": "7e002a08bef16abb5954d1c2a3fce0a2-574",
             },
             "@context": "https://raw.githubusercontent.com/dandi/schema/master/"
@@ -545,44 +555,23 @@ def test_aggregation_bids():
             "contentUrl": [
                 "https://api.dandiarchive.org/api/assets/6668d37f-e842-4b73-8c20"
                 "-082a1dd0d31a/download/",
-                "https://dandiarchive.s3.amazonaws.com/blobs/ace/16c/ace16cf2"
-                "-3eaf-4d64-9e3b-5c6a981cb8ec",
             ],
             "identifier": "6668d37f-e842-4b73-8c20-082a1dd0d31a",
             "repository": "https://dandiarchive.org/",
             "contentSize": 38474544973,
             "dateModified": "2021-07-22T23:59:16.060551-04:00",
             "schemaVersion": "0.4.4",
-            "encodingFormat": "application/x-hdf5",
-            "wasGeneratedBy": [
-                {
-                    "id": "urn:uuid:aef77d59-7a7f-4320-9d4b-9b03f3e25e54",
-                    "name": "Metadata generation",
-                    "schemaKey": "Activity",
-                    "description": "Metadata generated by DANDI cli",
-                    "wasAssociatedWith": [
-                        {
-                            "url": "https://github.com/dandi/dandi-cli",
-                            "name": "DANDI Command Line Interface",
-                            "version": "0.23.2",
-                            "schemaKey": "Software",
-                            "identifier": "RRID:SCR_019009",
-                        }
-                    ],
-                }
-            ],
+            "encodingFormat": "application/x-zarr",
             "blobDateModified": "2021-07-11T16:34:31-04:00",
         },
         {
             "id": "dandiasset:84dd580f-8d4a-43f8-bda3-6fb53fb5d3a2",
             "path": "sub-MITU01/ses-20210703h16m32s10/microscopy/sub-MITU01_"
-            "ses-20210703h16m32s10_run-1_sample-162_stain-LEC_chunk-5_spim.h5",
+            "ses-20210703h16m32s10_run-1_sample-162_stain-LEC_chunk-5_spim.ome.zarr",
             "access": [
                 {"status": "dandi:OpenAccess", "schemaKey": "AccessRequirements"}
             ],
             "digest": {
-                "dandi:sha2-256": "3d89ba29211c55c5a299f24ab64ac7d93750c8f7614"
-                "b360bf9a58266031b3752",
                 "dandi:dandi-etag": "954aab90c420c621cc70d17e74a65116-921",
             },
             "@context": "https://raw.githubusercontent.com/dandi/schema/master/"
@@ -591,34 +580,44 @@ def test_aggregation_bids():
             "contentUrl": [
                 "https://api.dandiarchive.org/api/assets/84dd580f-8d4a-43f8-bda3"
                 "-6fb53fb5d3a2/download/",
-                "https://dandiarchive.s3.amazonaws.com/blobs/3e4/c7f/3e4c7f03"
-                "-4280-426d-9cba-3b53d616a44c",
             ],
             "identifier": "84dd580f-8d4a-43f8-bda3-6fb53fb5d3a2",
             "repository": "https://dandiarchive.org/",
             "contentSize": 61774316916,
             "dateModified": "2021-10-01T18:28:16.038990-04:00",
             "schemaVersion": "0.6.0",
-            "encodingFormat": "application/x-hdf5",
-            "wasGeneratedBy": [
-                {
-                    "id": "urn:uuid:8f69a248-0e6a-4fa1-8369-ae1cc63d59d8",
-                    "name": "Metadata generation",
-                    "schemaKey": "Activity",
-                    "description": "Metadata generated by DANDI cli",
-                    "wasAssociatedWith": [
-                        {
-                            "url": "https://github.com/dandi/dandi-cli",
-                            "name": "DANDI Command Line Interface",
-                            "version": "0.27.3",
-                            "schemaKey": "Software",
-                            "identifier": "RRID:SCR_019009",
-                        }
-                    ],
-                }
-            ],
+            "encodingFormat": "application/x-zarr",
             "blobDateModified": "2021-07-10T18:58:25-04:00",
         },
+        {
+            "@context": "https://raw.githubusercontent.com/dandi/shortened",
+            "schemaKey": "Asset",
+            "repository": "https://dandiarchive.org/",
+            "dateModified": "2021-10-05T13:08:07.855880-04:00",
+            "schemaVersion": "0.6.0",
+            "encodingFormat": "application/json",
+            "blobDateModified": "2021-10-04T15:58:52.266222-04:00",
+            "id": "dandiasset:34e30fa6-cf6a-4a32-90cb-b06f6f2f30a6",
+            "access": [
+                {"schemaKey": "AccessRequirements", "status": "dandi:OpenAccess"}
+            ],
+            "path": "dataset_description.json",
+            "identifier": "34e30fa6-cf6a-4a32-90cb-b06f6f2f30a6",
+            "contentUrl": [
+                "https://api.dandiarchive.org/api/assets/shortened",
+            ],
+            "contentSize": 3377,
+            "digest": {
+                "dandi:dandi-etag": "88c82d75b1119393b9ee49adc14714f3-1",
+            },
+        },
     ]
     summary = aggregate_assets_summary(data)
+    assert summary["numberOfFiles"] == 3
     assert summary["numberOfSamples"] == 2
+    assert summary["numberOfSubjects"] == 1
+    assert sum("BIDS" in _.get("name", "") for _ in summary["dataStandard"]) == 1
+    assert (
+        sum(_.get("name", "").startswith("OME/NGFF") for _ in summary["dataStandard"])
+        == 1
+    )  # only a single entry so we do not duplicate them
