@@ -1,13 +1,16 @@
-import enum
-import json
+from collections import namedtuple
+from enum import Enum
+from inspect import isclass
+from typing import Any, Dict, List, Literal, Optional, Tuple, Type, Union, cast
 
 import pydantic
-from pydantic import ValidationError
+from pydantic import Field, ValidationError
 import pytest
 
-from .test_datacite import _basic_publishmeta
+from .utils import _basic_publishmeta
 from .. import models
 from ..models import (
+    DANDI_INSTANCE_URL_PATTERN,
     AccessRequirements,
     AccessType,
     Affiliation,
@@ -15,13 +18,12 @@ from ..models import (
     Asset,
     BaseType,
     CommonModel,
+    Contributor,
     DandiBaseModel,
     Dandiset,
     DigestType,
     IdentifierType,
     LicenseType,
-    List,
-    Optional,
     Organization,
     ParticipantRelationType,
     Person,
@@ -30,30 +32,31 @@ from ..models import (
     RelationType,
     Resource,
     RoleType,
-    Union,
 )
+from ..utils import TransitionalGenerateJsonSchema
 
 
-def test_dandiset():
-    assert Dandiset.unvalidated()
+def test_dandiset() -> None:
+    assert Dandiset.model_construct()  # type: ignore[call-arg]
 
 
-def test_asset():
-    assert Asset.unvalidated()
+def test_asset() -> None:
+    assert Asset.model_construct()  # type: ignore[call-arg]
 
 
-def test_asset_digest():
-    digest_model = {"sha1": ""}
+def test_asset_digest() -> None:
     with pytest.raises(pydantic.ValidationError) as exc:
         models.BareAsset(
-            contentSize=100, encodingFormat="nwb", digest=digest_model, path="/"
+            contentSize=100, encodingFormat="nwb", digest={"sha1": ""}, path="/"
         )
-    assert any(
-        [
-            "value is not a valid enumeration member" in val
-            for val in set([el["msg"] for el in exc.value.errors()])
-        ]
-    )
+
+    # Assert validation failed at the `digest` attribute because the provided dictionary
+    # key is not one of the DigestType enum values.
+    assert len(exc.value.errors()) == 1
+    err = exc.value.errors()[0]
+    assert err["type"] == "enum"
+    assert err["loc"] == ("digest", "sha1", "[key]")
+
     digest = 32 * "a"
     digest_model = {models.DigestType.dandi_etag: digest}
     with pytest.raises(pydantic.ValidationError) as exc:
@@ -71,23 +74,27 @@ def test_asset_digest():
     models.BareAsset(
         contentSize=100, encodingFormat="nwb", digest=digest_model, path="/"
     )
-    digest_model = {models.DigestType.dandi_etag: digest, "sha1": ""}
     with pytest.raises(pydantic.ValidationError) as exc:
-        models.PublishedAsset(
-            contentSize=100, encodingFormat="nwb", digest=digest_model, path="/"
+        models.PublishedAsset(  # type: ignore[call-arg]
+            contentSize=100,
+            encodingFormat="nwb",
+            digest={models.DigestType.dandi_etag: digest, "sha1": ""},
+            path="/",
         )
+
+    # Assert validation failed at the `digest` attribute because the provided dictionary
+    # key is not one of the DigestType enum values.
     assert any(
-        [
-            "value is not a valid enumeration member" in val
-            for val in set([el["msg"] for el in exc.value.errors()])
-        ]
+        err["type"] == "enum" and err["loc"] == ("digest", "sha1", "[key]")
+        for err in exc.value.errors()
     )
+
     digest_model = {
         models.DigestType.dandi_etag: digest,
         models.DigestType.sha2_256: 63 * "a",
     }
     with pytest.raises(pydantic.ValidationError) as exc:
-        models.PublishedAsset(
+        models.PublishedAsset(  # type: ignore[call-arg]
             contentSize=100, encodingFormat="nwb", digest=digest_model, path="/"
         )
     assert any(
@@ -99,7 +106,7 @@ def test_asset_digest():
         models.DigestType.sha2_256: 64 * "a",
     }
     with pytest.raises(pydantic.ValidationError) as exc:
-        models.PublishedAsset(
+        models.PublishedAsset(  # type: ignore[call-arg]
             contentSize=100, encodingFormat="nwb", digest=digest_model, path="/"
         )
     assert not any(
@@ -110,7 +117,7 @@ def test_asset_digest():
         models.DigestType.dandi_etag: digest,
     }
     with pytest.raises(pydantic.ValidationError) as exc:
-        models.PublishedAsset(
+        models.PublishedAsset(  # type: ignore[call-arg]
             contentSize=100, encodingFormat="nwb", digest=digest_model, path="/"
         )
     assert any(
@@ -118,7 +125,7 @@ def test_asset_digest():
         for el in exc.value.errors()
     )
 
-    digest = 33 * "a"
+    digest = 32 * "a"
     digest_model = {models.DigestType.dandi_zarr_checksum: digest}
     with pytest.raises(pydantic.ValidationError) as exc:
         models.BareAsset(
@@ -133,27 +140,31 @@ def test_asset_digest():
             for val in set([el["msg"] for el in exc.value.errors()])
         ]
     )
-    digest = 32 * "a"
+    digest = f"{32 * 'a'}-1--42"
     digest_model = {models.DigestType.dandi_zarr_checksum: digest}
-    models.BareAsset(
-        contentSize=100,
-        encodingFormat="application/x-zarr",
-        digest=digest_model,
-        path="/",
-    )
     with pytest.raises(pydantic.ValidationError) as exc:
-        models.PublishedAsset(
+        models.BareAsset(
             contentSize=100,
             encodingFormat="application/x-zarr",
             digest=digest_model,
             path="/",
         )
-    assert all(
+    assert any(
         [
-            "field required" in val
+            "contentSize 100 is not equal to the checksum size 42." in val
             for val in set([el["msg"] for el in exc.value.errors()])
         ]
     )
+    digest = f"{32 * 'a'}-1--100"
+    digest_model = {models.DigestType.dandi_zarr_checksum: digest}
+    with pytest.raises(pydantic.ValidationError) as exc:
+        models.PublishedAsset(  # type: ignore[call-arg]
+            contentSize=100,
+            encodingFormat="application/x-zarr",
+            digest=digest_model,
+            path="/",
+        )
+    assert all(err["type"] == "missing" for err in exc.value.errors())
     digest_model = {
         models.DigestType.dandi_zarr_checksum: digest,
         models.DigestType.dandi_etag: digest + "-1",
@@ -172,7 +183,7 @@ def test_asset_digest():
         ]
     )
     with pytest.raises(pydantic.ValidationError) as exc:
-        models.PublishedAsset(
+        models.PublishedAsset(  # type: ignore[call-arg]
             contentSize=100,
             encodingFormat="application/x-zarr",
             digest=digest_model,
@@ -199,7 +210,7 @@ def test_asset_digest():
         ]
     )
     with pytest.raises(pydantic.ValidationError) as exc:
-        models.PublishedAsset(
+        models.PublishedAsset(  # type: ignore[call-arg]
             contentSize=100,
             encodingFormat="application/x-zarr",
             digest=digest_model,
@@ -345,21 +356,21 @@ def test_asset_digest():
         ),
     ],
 )
-def test_types(enumtype, values):
+def test_types(enumtype: Type[Enum], values: Dict[str, str]) -> None:
     assert {v.name: v.value for v in enumtype} == values
 
 
-def test_autogenerated_titles():
-    schema = Asset.schema()
+def test_autogenerated_titles() -> None:
+    schema = Asset.model_json_schema(schema_generator=TransitionalGenerateJsonSchema)
     assert schema["title"] == "Asset"
     assert schema["properties"]["schemaVersion"]["title"] == "Schema Version"
-    assert schema["definitions"]["PropertyValue"]["title"] == "Property Value"
+    assert schema["$defs"]["PropertyValue"]["title"] == "Property Value"
 
 
-def test_dantimeta_1():
-    """ checking basic metadata for publishing"""
+def test_dantimeta_1() -> None:
+    """checking basic metadata for publishing"""
     # meta data without doi, datePublished and publishedBy
-    meta_dict = {
+    meta_dict: Dict[str, Any] = {
         "identifier": "DANDI:999999",
         "id": "DANDI:999999/draft",
         "version": "1.0.0",
@@ -368,7 +379,9 @@ def test_dantimeta_1():
         "contributor": [
             {
                 "name": "last name, first name",
+                "email": "someone@dandiarchive.org",
                 "roleName": [RoleType("dcite:ContactPerson")],
+                "schemaKey": "Person",
             }
         ],
         "license": [LicenseType("spdx:CC-BY-4.0")],
@@ -392,14 +405,38 @@ def test_dantimeta_1():
     with pytest.raises(ValidationError) as exc:
         PublishedDandiset(**meta_dict)
 
-    error_msgs = [
-        "field required",
-        "A Dandiset containing no files or zero bytes is not publishable",
-        'string does not match regex "^https://dandiarchive.org/dandiset/'
-        '\\d{6}/\\d+\\.\\d+\\.\\d+$"',
-        'string does not match regex "^DANDI:\\d{6}/\\d+\\.\\d+\\.\\d+"',
-    ]
-    assert all([el["msg"] in error_msgs for el in exc.value.errors()])
+    ErrDetail = namedtuple("ErrDetail", ["type", "msg"])
+
+    # Expected errors keyed by location of the respective error
+    # Note: Pydantic generated error messages are not provided for they are not in our
+    #       control, and the error type should be indicative enough.
+    expected_errors: Dict[Tuple[Union[int, str], ...], ErrDetail] = {
+        ("id",): ErrDetail(type="string_pattern_mismatch", msg=None),
+        ("publishedBy",): ErrDetail(type="missing", msg=None),
+        ("datePublished",): ErrDetail(type="missing", msg=None),
+        ("url",): ErrDetail(
+            type="value_error",
+            msg="Value error, string does not match regex "
+            f'"^{DANDI_INSTANCE_URL_PATTERN}/dandiset/'
+            '\\d{6}/\\d+\\.\\d+\\.\\d+$"',
+        ),
+        ("assetsSummary",): ErrDetail(
+            type="value_error",
+            msg="Value error, "
+            "A Dandiset containing no files or zero bytes is not publishable",
+        ),
+        ("doi",): ErrDetail(type="missing", msg=None),
+    }
+
+    assert len(exc.value.errors()) == 6
+    for err in exc.value.errors():
+        err_loc = err["loc"]
+        assert err_loc in expected_errors
+
+        assert err["type"] == expected_errors[err_loc].type
+        if expected_errors[err_loc].msg is not None:
+            assert err["msg"] == expected_errors[err_loc].msg
+
     assert set([el["loc"][0] for el in exc.value.errors()]) == {
         "assetsSummary",
         "datePublished",
@@ -419,7 +456,7 @@ def test_dantimeta_1():
     PublishedDandiset(**meta_dict)
 
 
-def test_schemakey():
+def test_schemakey() -> None:
     typemap = {
         "BareAsset": "Asset",
         "PublishedAsset": "Asset",
@@ -429,18 +466,18 @@ def test_schemakey():
         if val in ["BaseModel"]:
             continue
         klass = getattr(models, val)
-        if isinstance(klass, pydantic.main.ModelMetaclass):
-            assert "schemaKey" in klass.__fields__
+        if isclass(klass) and issubclass(klass, pydantic.BaseModel):
+            assert "schemaKey" in klass.model_fields
             if val in typemap:
-                assert typemap[val] == klass.__fields__["schemaKey"].default
+                assert typemap[val] == klass.model_fields["schemaKey"].default
             else:
-                assert val == klass.__fields__["schemaKey"].default
+                assert val == klass.model_fields["schemaKey"].default
 
 
-def test_duplicate_classes():
-    qnames = {}
+def test_duplicate_classes() -> None:
+    qnames: Dict[str, Optional[type]] = {}
 
-    def check_qname(qname, klass):
+    def check_qname(qname: str, klass: type) -> None:
         if (
             qname
             in [
@@ -452,11 +489,12 @@ def test_duplicate_classes():
         ):
             return
         if qname in qnames:
-            if qnames[qname] is None:
+            t = qnames[qname]
+            if t is None:
                 return
-            if issubclass(klass, (qnames[qname],)):
+            elif issubclass(klass, (t,)):
                 return
-            if issubclass(qnames[qname], klass):
+            elif issubclass(t, klass):
                 qnames[qname] = klass
                 return
             if qname == "dandi:repository" and klass.__name__ in (
@@ -489,28 +527,27 @@ def test_duplicate_classes():
     modelnames.remove("DandiBaseModel")
     for val in ["CommonModel", "BaseType"] + modelnames:
         klass = getattr(models, val)
-        if not isinstance(klass, pydantic.main.ModelMetaclass):
+        if not isclass(klass) or not issubclass(klass, pydantic.BaseModel):
             continue
-        if isinstance(klass, enum.EnumMeta):
-            for enumval in klass:
-                qname = enumval.value
-                check_qname(qname, klass)
         if hasattr(klass, "_ldmeta"):
-            if "nskey" in klass._ldmeta:
+            if "nskey" in klass._ldmeta.default:
                 name = klass.__name__
-                qname = f'{klass._ldmeta["nskey"]}:{name}'
+                qname = f'{klass._ldmeta.default["nskey"]}:{name}'
             else:
                 qname = f"dandi:{name}"
             check_qname(qname, klass)
-        for name, field in klass.__fields__.items():
-            if "nskey" in field.field_info.extra:
-                qname = field.field_info.extra["nskey"] + ":" + name
+        for name, field in klass.model_fields.items():
+            if (
+                isinstance(field.json_schema_extra, dict)
+                and "nskey" in field.json_schema_extra
+            ):
+                qname = cast(str, field.json_schema_extra["nskey"]) + ":" + name
             else:
                 qname = f"dandi:{name}"
             check_qname(qname, klass)
 
 
-def test_properties_mismatch():
+def test_properties_mismatch() -> None:
     prop_names = {}
     errors = []
     modelnames = dir(models)
@@ -521,12 +558,16 @@ def test_properties_mismatch():
     modelnames.remove("Publishable")
     for val in modelnames:
         klass = getattr(models, val)
-        if not isinstance(klass, pydantic.main.ModelMetaclass):
+        if not isclass(klass) or not issubclass(klass, pydantic.BaseModel):
             continue
-        if not hasattr(klass, "_ldmeta") or "nskey" not in klass._ldmeta:
+        if not hasattr(klass, "_ldmeta") or "nskey" not in klass._ldmeta.default:
             errors.append(f"{klass} does not have nskey")
-        for name, field in klass.__fields__.items():
-            nskey = field.field_info.extra.get("nskey", "dandi")
+        for name, field in klass.model_fields.items():
+            if field.json_schema_extra is None:
+                nskey = "dandi"
+            else:
+                assert isinstance(field.json_schema_extra, dict)
+                nskey = cast(str, field.json_schema_extra.get("nskey", "dandi"))
             if name not in prop_names:
                 prop_names[name] = nskey
             elif nskey != prop_names[name]:
@@ -536,10 +577,14 @@ def test_properties_mismatch():
     assert errors == []
 
 
-def test_schemakey_roundtrip():
-    class TempKlass(DandiBaseModel):
-        contributor: Optional[List[Union[Organization, Person]]]
+class TempKlass1(DandiBaseModel):
+    contributor: Optional[List[Union[Organization, Person]]] = None
+    schemaKey: Literal["TempKlass1"] = Field(
+        "TempKlass1", validate_default=True, json_schema_extra={"readOnly": True}
+    )
 
+
+def test_schemakey_roundtrip() -> None:
     contributor = [
         {
             "name": "first",
@@ -550,6 +595,7 @@ def test_schemakey_roundtrip():
         },
         {
             "name": "last2, first2",
+            "email": "nospam@dandiarchive.org",
             "roleName": ["dcite:ContactPerson"],
             "schemaKey": "Person",
             "affiliation": [],
@@ -557,20 +603,26 @@ def test_schemakey_roundtrip():
         },
     ]
     with pytest.raises(pydantic.ValidationError):
-        TempKlass(contributor=contributor)
+        TempKlass1(contributor=contributor)
     contributor[0]["name"] = ", "
     with pytest.raises(pydantic.ValidationError):
-        TempKlass(contributor=contributor)
+        TempKlass1(contributor=contributor)
     contributor[0]["name"] = "last, first"
-    klassobj = TempKlass(contributor=contributor)
-    assert all([isinstance(val, Person) for val in klassobj.contributor])
+    klassobj = TempKlass1(contributor=contributor)
+    assert klassobj.contributor is not None and all(
+        [isinstance(val, Person) for val in klassobj.contributor]
+    )
+
+
+class TempKlass2(DandiBaseModel):
+    contributor: Person
+    schemaKey: Literal["TempKlass2"] = Field(
+        "TempKlass2", validate_default=True, json_schema_extra={"readOnly": True}
+    )
 
 
 @pytest.mark.parametrize("name", ["Mitášová, Helena", "O'Brien, Claire"])
-def test_name_regex(name):
-    class TempKlass(DandiBaseModel):
-        contributor: Person
-
+def test_name_regex(name: str) -> None:
     contributor = {
         "name": name,
         "roleName": [],
@@ -578,18 +630,20 @@ def test_name_regex(name):
         "affiliation": [],
         "includeInCitation": True,
     }
-    TempKlass(contributor=contributor)
+    TempKlass2(contributor=contributor)
 
 
-def test_resource():
+def test_resource() -> None:
     with pytest.raises(pydantic.ValidationError):
         Resource(relation=RelationType.IsCitedBy)
     Resource(identifier="123", relation=RelationType.IsCitedBy)
     Resource(url="http://example.org/resource", relation=RelationType.IsCitedBy)
 
 
-def test_basetype():
-    props = json.loads(BaseType.schema_json())["properties"]
+def test_basetype() -> None:
+    props = BaseType.model_json_schema(schema_generator=TransitionalGenerateJsonSchema)[
+        "properties"
+    ]
     identifier = props["identifier"]
     assert "anyOf" not in identifier
     assert identifier.get("maxLength") == 1000
@@ -597,35 +651,39 @@ def test_basetype():
     assert key["const"] == "BaseType"
 
 
-def test_https_regex():
-    props = json.loads(Affiliation.schema_json())["properties"]["identifier"]
+def test_https_regex() -> None:
+    props = Affiliation.model_json_schema(
+        schema_generator=TransitionalGenerateJsonSchema
+    )["properties"]["identifier"]
     assert props["format"] == "uri"
     assert props.get("maxLength") == 1000
 
 
-def test_schemakey_in_required():
-    props = json.loads(Affiliation.schema_json())["required"]
+def test_schemakey_in_required() -> None:
+    props = Affiliation.model_json_schema(
+        schema_generator=TransitionalGenerateJsonSchema
+    )["required"]
     assert "schemaKey" in props
 
 
 @pytest.mark.parametrize("value", [None, [], {}, (), ""])
-def test_propertyvalue(value):
+def test_propertyvalue(value: Any) -> None:
     with pytest.raises(pydantic.ValidationError):
         PropertyValue(value=value)
 
 
-def test_propertyvalue_valid():
+def test_propertyvalue_valid() -> None:
     PropertyValue(value=1)
 
 
-def test_propertyvalue_json():
-    reqprops = json.loads(PropertyValue.schema_json())["definitions"]["PropertyValue"][
-        "required"
-    ]
+def test_propertyvalue_json() -> None:
+    reqprops = PropertyValue.model_json_schema(
+        schema_generator=TransitionalGenerateJsonSchema
+    )["$defs"]["PropertyValue"]["required"]
     assert "value" == reqprops[1]
 
 
-def test_embargoedaccess():
+def test_embargoedaccess() -> None:
     with pytest.raises(pydantic.ValidationError):
         CommonModel(access=[AccessRequirements(status=AccessType.EmbargoedAccess)])
     CommonModel(
@@ -635,3 +693,43 @@ def test_embargoedaccess():
             )
         ]
     )
+
+
+_NON_CONTACT_PERSON_ROLES_ARGS: List[List[RoleType]] = [
+    [],
+    [RoleType.Author, RoleType.DataCurator],
+    [RoleType.Funder],
+]
+
+_CONTACT_PERSON_ROLES_ARGS: List[List[RoleType]] = [
+    role_lst + [RoleType.ContactPerson] for role_lst in _NON_CONTACT_PERSON_ROLES_ARGS
+]
+
+
+class TestContributor:
+
+    @pytest.mark.parametrize("roles", _CONTACT_PERSON_ROLES_ARGS)
+    def test_contact_person_without_email(self, roles: List[RoleType]) -> None:
+        """
+        Test creating a `Contributor` instance as a contact person without an email
+        """
+        with pytest.raises(
+            pydantic.ValidationError, match="Contact person must have an email address"
+        ):
+            Contributor(roleName=roles)
+
+    @pytest.mark.parametrize("roles", _NON_CONTACT_PERSON_ROLES_ARGS)
+    def test_non_contact_person_without_email(self, roles: List[RoleType]) -> None:
+        """
+        Test creating a `Contributor` instance as a non-contact person without an email
+        """
+        Contributor(roleName=roles)
+
+    @pytest.mark.parametrize(
+        "roles", _NON_CONTACT_PERSON_ROLES_ARGS + _CONTACT_PERSON_ROLES_ARGS
+    )
+    def test_with_email(self, roles: List[RoleType]) -> None:
+        """
+        Test creating a `Contributor` instance with an email
+        """
+        Contributor(email="nemo@dandiarchive.org", roleName=roles)
